import os

import click

from .. import settings
from .common import validate_package_config, load_config


BOILERPLATE_REQUIRED_FILES = (
    'templates/base.html',
)

BOILERPLATE_REQUIRED_CONFIG_KEYS = (
    'package-name',
    'identifier',
    'version',
    'templates',
)


def validate_templates(config):
    data = config.get('templates', [])
    if isinstance(data, (list, tuple)):
        for template in data:
            if not isinstance(template, (list, tuple)) or len(template) != 2:
                return (
                    'Templates must be a list/tuple of lists/tuples '
                    'with two items each.'
                )


def validate_protected_files(config, path):
    protected_files = config.get('protected', [])
    if not isinstance(protected_files, (list, tuple)):
        return ['Protected files setting must be a list or a tuple.']

    errors = []
    for fname in protected_files:
        fpath = os.path.join(path, fname)
        if not fpath.endswith('*') and not os.path.exists(fpath):
            errors.append('Protected file "{}" not found'.format(fname))

    return errors


def validate_boilerplate_config(config, path):
    missing_files = []
    for required_file in BOILERPLATE_REQUIRED_FILES:
        fpath = os.path.join(path, required_file)
        if not os.path.exists(fpath):
            missing_files.append(fpath)

    if missing_files:
        click.secho(
            'The following required files were not found:\n{}'.format(
                '\n'.join(missing_files)),
            fg='yellow')

    errors = validate_package_config(
        config=config,
        required_keys=BOILERPLATE_REQUIRED_CONFIG_KEYS,
        config_path=path
    )

    template_validation_error = validate_templates(config)
    if template_validation_error:
        errors.append(template_validation_error)

    protected_files_validation_errors = validate_protected_files(config, path)
    errors.extend(protected_files_validation_errors)

    if errors:
        message = 'The following errors happened during validation:'
        message = '{}\n - {}'.format(message, '\n - '.join(errors))
        raise click.ClickException(message)

    return True


def validate_boilerplate(path=None):
    config = load_config(settings.BOILERPLATE_CONFIG_FILENAME, path)
    return validate_boilerplate_config(config, path)



SESSION_EXPIRED = 'Session expired. Please log in again.'
NETWORK_ERROR_MESSAGE = (
    'Network error. Please check your connection and try again.'
)
AUTH_SERVER_ERROR = (
    'A problem occured while trying to authenticate with aldryn.com. '
    'Please try again later'
)
SERVER_ERROR = (
    'A problem occured while trying to communicate with aldryn.com. '
    'Please try again later'
)
AUTH_INVALID_TOKEN = 'Login failed. Invalid token specified'
RESOURCE_NOT_FOUND = 'Requested resource could not be found'
LOGIN_SUCCESSFUL = 'Welcome to Aldryn. You are now logged in as {greeting}'
CONFIG_FILE_NOT_FOUND = 'Config file could not be not found at location: {}'
FILE_NOT_FOUND = 'File could not be found: {}'
INVALID_DB_SUBMITTED = (
    "The database dump you have uploaded contains an error. "
    "Please check the file 'db_upload.log' for errors and try again"
)



import json
import re
import os

import click


from .. import messages


VALID_LICENSE_FILENAMES = (
    'LICENSE.txt',
    'LICENSE',
    'license.txt',
    'license',
)

FILENAME_BASIC_RE = re.compile(
    r'^[a-zA-Z0-9_@]+[a-zA-Z0-9._@-]*\.[a-zA-Z0-9]{1,23}$'
)

ALLOWED_EXTENSIONS = (
    '.css',
    '.gif',
    '.htc',
    '.htm',
    '.html',
    '.ico',
    '.jpeg',
    '.jpg',
    '.js',
    '.json',
    '.less',
    '.map',
    '.png',
    '.rb',
    '.sass',
    '.scss',
    '.svg',
    '.webm',
    # font formats
    '.eot',
    '.ttf',
    '.woff',
    '.woff2',
    '.otf',
    # text formats
    '.txt',
    '.md',
    '.rst',
    # document formats
    '.pdf',
    '.ps',
    '.djv',
    '.djvu',
    '.doc',
    '.docx',
    '.ppt',
    '.pptx',
    '.xls',
    '.xlsx',
)


def is_valid_file_name(name):
    if not FILENAME_BASIC_RE.match(name):
        return False
    ext = os.path.splitext(name)[-1]
    if ext not in ALLOWED_EXTENSIONS:
        return False
    return True


def get_license(path):
    for fname in VALID_LICENSE_FILENAMES:
        fpath = os.path.join(path, fname)
        if os.path.exists(fpath):
            return fpath


def load_config(fname, path=None):
    config_fpath = os.path.join(path or '.', fname)
    if not os.path.exists(config_fpath):
        raise click.ClickException(
            messages.FILE_NOT_FOUND.format(config_fpath)
        )

    with open(config_fpath) as f:
        try:
            return json.load(f)
        except ValueError:
            raise click.ClickException(
                'Config file could not be loaded: Invalid JSON'
            )


def validate_package_config(config, required_keys, config_path):
    errors = []

    if not get_license(config_path):
        errors.append(
            'Required LICENSE file not found. Valid names are {}.'.format(
                ', '.join(VALID_LICENSE_FILENAMES)
            )
        )

    for key in required_keys:
        if key not in config:
            errors.append(
                'Required key "{}" not found in config file.'.format(key)
            )

    return errors



import os
import re
from netrc import netrc
from time import sleep

import click
from six.moves.urllib_parse import urlparse

from . import settings
from . import messages
from . import api_requests
from .config import Config


ENDPOINT = 'https://control.{host}'
DEFAULT_HOST = 'aldryn.com'


def get_aldryn_host():
    return os.environ.get('ALDRYN_HOST', DEFAULT_HOST)


def get_endpoint(host=None):
    if not host:
        host = get_aldryn_host()
    if re.match('^https?://', host):
        endpoint = host
    else:
        endpoint = ENDPOINT.format(host=host)

    if host != DEFAULT_HOST:
        click.secho('Using custom endpoint {}\n'.format(endpoint), fg='yellow')
    return endpoint


class CloudClient(object):
    def __init__(self, endpoint):
        self.config = Config()
        self.endpoint = endpoint
        self.netrc = WritableNetRC()
        self.session = self.init_session()

    # Helpers
    def get_auth_header(self):
        host = urlparse(self.endpoint).hostname
        data = self.netrc.hosts.get(host)
        if data:
            return {'Authorization': 'Basic {}'.format(data[2])}
        return {}

    def get_access_token_url(self):
        return '{}/{}'.format(
            self.endpoint.rstrip('/'),
            settings.ACCESS_TOKEN_URL_PATH.lstrip('/')
        )

    def init_session(self):
        return api_requests.SingleHostSession(
            self.endpoint,
            headers=self.get_auth_header(),
            trust_env=False
        )

    def authenticate(self, token):
        self.session.headers['Authorization'] = 'Basic {}'.format(token)

    def login(self, token):
        request = api_requests.LoginRequest(
            self.session,
            data={'token': token}
        )
        user_data = request()

        self.authenticate(token)

        first_name = user_data.get('first_name')
        last_name = user_data.get('last_name')
        email = user_data.get('email')

        if first_name and last_name:
            greeting = '{} {} ({})'.format(first_name, last_name, email)
        elif first_name:
            greeting = '{} ({})'.format(first_name, email)
        else:
            greeting = email

        self.netrc.add(urlparse(self.endpoint).hostname, email, None, token)
        self.netrc.write()

        return messages.LOGIN_SUCCESSFUL.format(greeting=greeting)

    def get_projects(self):
        request = api_requests.ProjectListRequest(self.session)
        return request()

    def deploy_project_or_get_progress(self, website_id, stage):
        def fmt_progress(data):
            if not data:
                return 'Connecting to remote'
            return '{} ({})'.format(
                data['verbose_state'],
                data['heartbeat_ago_formatted']
            )

        response = self.deploy_project_progress(website_id, stage)
        if response['is_deploying']:
            click.secho(
                'Already deploying {} server, attaching to running '
                'deployment'.format(stage),
                fg='yellow'
            )
        else:
            click.secho('Deploying {} server'.format(stage), fg='green')
            self.deploy_project(website_id, stage)
            sleep(1)
            response = self.deploy_project_progress(website_id, stage)
        try:
            with click.progressbar(
                    length=100, show_percent=True,
                    show_eta=False, item_show_func=fmt_progress) as bar:
                while response['is_deploying']:
                    response = self.deploy_project_progress(website_id, stage)
                    bar.current_item = progress = response['deploy_progress']
                    bar.update(
                        # update the difference of the current percentage
                        # to the new percentage
                        progress['main_percent'] +
                        progress['extra_percent'] -
                        bar.pos
                    )
                    sleep(3)
        except KeyboardInterrupt:
            click.secho('Disconnected')

    def deploy_project_progress(self, website_id, stage):
        request = api_requests.DeployProjectProgressRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        data = request()
        return data[stage]

    def deploy_project(self, website_id, stage):
        request = api_requests.DeployProjectRequest(
            self.session,
            url_kwargs={'website_id': website_id},
            data={'stage': stage}
        )
        return request()

    def get_project(self, website_id):
        request = api_requests.ProjectDetailRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def is_project_locked(self, website_id):
        request = api_requests.ProjectLockQueryRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def lock_project(self, website_id):
        request = api_requests.ProjectLockRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def unlock_project(self, website_id):
        request = api_requests.ProjectUnlockRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def register_addon(self, package_name, verbose_name, organisation_id=None):
        request = api_requests.RegisterAddonRequest(
            self.session,
            data={
                'package_name': package_name,
                'name': verbose_name,
                'organisation': organisation_id,
            }
        )
        return request()

    def upload_addon(self, archive_obj):
        request = api_requests.UploadAddonRequest(
            self.session,
            files={'app': archive_obj}
        )
        return request()

    def upload_boilerplate(self, archive_obj):
        request = api_requests.UploadBoilerplateRequest(
            self.session,
            files={'boilerplate': archive_obj}
        )
        return request()

    def get_website_id_for_slug(self, slug):
        request = api_requests.SlugToIDRequest(
            self.session,
            url_kwargs={'website_slug': slug}
        )
        return request()

    def get_website_slug_for_id(self, website_id):
        request = api_requests.IDToSlugRequest(
            self.session,
            url_kwargs={'website_id': website_id}
        )
        return request()

    def download_backup(self, website_slug, filename=None, directory=None):
        request = api_requests.DownloadBackupRequest(
            self.session,
            url_kwargs={'website_slug': website_slug},
            filename=filename,
            directory=directory,
        )
        return request()

    def download_db_request(self, website_id):
        request = api_requests.DownloadDBRequestRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def download_db_progress(self, url):
        request = api_requests.DownloadDBProgressRequest(
            self.session,
            url=url,
        )
        return request()

    def download_db(self, website_slug, url=None, filename=None, directory=None):
        request = api_requests.DownloadDBRequest(
            self.session,
            url=url,
            url_kwargs={'website_slug': website_slug},
            filename=filename,
            directory=directory,
        )
        return request()

    def download_media_request(self, website_id):
        request = api_requests.DownloadMediaRequestRequest(
            self.session,
            url_kwargs={'website_id': website_id},
        )
        return request()

    def download_media_progress(self, url):
        request = api_requests.DownloadMediaProgressRequest(
            self.session,
            url=url,
        )
        return request()

    def download_media(self, website_slug, url=None,
                       filename=None, directory=None):
        request = api_requests.DownloadMediaRequest(
            self.session,
            url=url,
            url_kwargs={'website_slug': website_slug},
            filename=filename,
            directory=directory,
        )
        return request()

    def upload_db(self, website_id, archive_path):
        request = api_requests.UploadDBRequest(
            self.session,
            url_kwargs={'website_id': website_id},
            files={'db_dump': open(archive_path, 'rb')}
        )
        return request()

    def upload_db_progress(self, url):
        request = api_requests.UploadDBProgressRequest(
            self.session,
            url=url,
        )
        return request()

    def upload_media(self, website_id, archive_path):
        request = api_requests.UploadMediaFilesRequest(
            self.session,
            url_kwargs={'website_id': website_id},
            files={'media_files': open(archive_path, 'rb')}
        )
        return request()

    def upload_media_progress(self, url):
        request = api_requests.UploadMediaFilesProgressRequest(
            self.session,
            url=url,
        )
        return request()


class WritableNetRC(netrc):
    def __init__(self, *args, **kwargs):
        netrc_path = self.get_netrc_path()
        if not os.path.exists(netrc_path):
            open(netrc_path, 'a').close()
            os.chmod(netrc_path, 0o600)
        kwargs['file'] = netrc_path
        try:
            netrc.__init__(self, *args, **kwargs)
        except IOError:
            raise click.ClickException(
                "Please make sure your netrc config file ('{}') can be read "
                "and written by the current user."
                .format(netrc_path)
            )

    def get_netrc_path(self):
        """
        netrc uses os.environ['HOME'] for path detection which is
        not defined on Windows. Detecting the correct path ourselves
        """
        home = os.path.expanduser('~')
        return os.path.join(home, '.netrc')

    def add(self, host, login, account, password):
        self.hosts[host] = (login, account, password)

    def remove(self, host):
        if host in self.hosts:
            del self.hosts[host]

    def write(self, path=None):
        if path is None:
            path = self.get_netrc_path()

        out = []
        for machine, data in self.hosts.items():
            login, account, password = data
            out.append('machine {}'.format(machine))
            if login:
                out.append('\tlogin {}'.format(login))
            if account:
                out.append('\taccount {}'.format(account))
            if password:
                out.append('\tpassword {}'.format(password))

        with open(path, 'w') as f:
            f.write(os.linesep.join(out))



import subprocess
import itertools
import os
import sys
from distutils.version import StrictVersion

try:
    import ipdb as pdb
except ImportError:
    import pdb

import click

from . import localdev
from .localdev.utils import get_aldryn_project_settings
from .cloud import CloudClient, get_endpoint
from .check_system import check_requirements
from .utils import (
    hr, table, open_project_cloud_site, get_dashboard_url,
    get_project_cheatsheet_url, get_latest_version_from_pypi,
)
from .validators.addon import validate_addon
from .validators.boilerplate import validate_boilerplate
from .upload.addon import upload_addon
from .upload.boilerplate import upload_boilerplate


@click.group()
@click.option('-d', '--debug/--no-debug', default=False,
              help=('Drop into the debugger if the command execution raises '
                    'an exception.'))
@click.pass_context
def cli(ctx, debug):
    if debug:
        def exception_handler(type, value, traceback):
            click.secho(
                '\nAn exception occurred while executing the requested '
                'command:', fg='red'
            )
            hr(fg='red')
            sys.__excepthook__(type, value, traceback)
            click.secho('\nStarting interactive debugging session:', fg='red')
            hr(fg='red')
            pdb.post_mortem(traceback)
        sys.excepthook = exception_handler

    ctx.obj = CloudClient(get_endpoint())

    # skip if 'aldryn version' is run
    if not ctx.args == ['version']:
        # check for newer versions
        ctx.obj.config.check_for_updates()


def login_token_helper(ctx, param, value):
    if not value:
        url = ctx.obj.get_access_token_url()
        click.secho('Your browser has been opened to visit: {}'.format(url))
        click.launch(url)
        value = click.prompt('Please copy the access token and paste it here')
    return value


@cli.command()
@click.argument('token', required=False, callback=login_token_helper)
@click.pass_obj
def login(obj, token):
    """Authorize your machine with Aldryn"""
    click.echo(obj.login(token))


@cli.group()
def project():
    """Manage your projects"""
    pass


@project.command(name='list')
@click.option(
    '-g', '--grouped', is_flag=True, default=False,
    help='Group by organisation'
)
@click.pass_obj
def project_list(obj, grouped):
    """List all your projects"""
    api_response = obj.get_projects()
    header = ('Slug', 'Name', 'Organisation')

    # get all users + organisations
    groups = {
        'users': {
            account['id']: {'name': 'Personal', 'projects': []}
            for account in api_response['accounts']
            if account['type'] == 'user'
        },
        'organisations': {
            account['id']: {'name': account['name'], 'projects': []}
            for account in api_response['accounts']
            if account['type'] == 'organisation'
        }
    }

    # sort websites into groups
    for website in api_response['websites']:
        organisation_id = website['organisation_id']
        if organisation_id:
            owner = groups['organisations'][website['organisation_id']]
        else:
            owner = groups['users'][website['owner_id']]
        owner['projects'].append((website['domain'], website['name']))

    accounts = itertools.chain(
        groups['users'].items(),
        groups['organisations'].items()
    )

    def sort_projects(items):
        return sorted(items, key=lambda x: x[0].lower())

    # print via pager
    if grouped:
        output_items = []
        for group, data in accounts:
            projects = data['projects']
            if projects:
                output_items.append(
                    u'{title}\n{line}\n\n{table}\n\n'.format(
                        title=data['name'],
                        line='=' * len(data['name']),
                        table=table(sort_projects(projects), header[:2])
                    )
                )
        output = os.linesep.join(output_items).rstrip(os.linesep)
    else:
        # add account name to all projects
        projects = [
            each + (data['name'],) for group, data in accounts
            for each in data['projects']
        ]
        output = table(sort_projects(projects), header)

    click.echo_via_pager(output)


# @project.command(name='info')
# @click.argument('slug')
# @click.pass_obj
# def project_info(obj, slug):
#     """Show info about a project"""
#     # TODO: proper formatting
#     website_id = obj.get_website_id_for_slug(slug)
#     click.echo(obj.get_project(website_id))


@project.command(name='deploy')
@click.argument('stage', default='test')
@click.pass_obj
def project_deploy(obj, stage):
    """Deploy project"""
    website_id = get_aldryn_project_settings()['id']
    obj.deploy_project_or_get_progress(website_id, stage)


@project.command(name='dashboard')
@click.pass_obj
def project_dashboard(obj):
    """Open project dashboard"""
    click.launch(get_dashboard_url(obj))


@project.command(name='up')
@click.pass_obj
def project_up(obj):
    """Start local project"""
    localdev.start_project()


@project.command(name='open')
@click.pass_obj
def project_open(obj):
    """Open local project in browser"""
    localdev.open_project()


@project.command(name='update')
@click.pass_obj
def project_update(obj):
    """Update project with latest changes from the Cloud"""
    localdev.update_local_project()


@project.command(name='test')
@click.pass_obj
def project_open_test(obj):
    """Open project test site"""
    open_project_cloud_site(obj, 'test')


@project.command(name='live')
@click.pass_obj
def project_open_live(obj):
    """Open project live site"""
    open_project_cloud_site(obj, 'live')


@project.command(name='status')
@click.pass_obj
def project_status(obj):
    """Show local project status"""
    localdev.show_project_status()


@project.command(name='stop')
@click.pass_obj
def project_stop(obj):
    """Stop local project"""
    localdev.stop_project()


@project.command(name='cheatsheet')
@click.pass_obj
def project_cheatsheet(obj):
    """Show useful commands for your project"""
    click.launch(get_project_cheatsheet_url(obj))


@project.command(name='setup')
@click.argument('slug')
@click.option(
    '-p', '--path', default='.', help='install project to path',
    type=click.Path(writable=True, readable=True)
)
@click.pass_obj
def project_setup(obj, slug, path):
    """Set up a development environment for an Aldryn project"""
    if not check_requirements(silent=True):
        click.secho(
            "There was a problem while checking your system. Please run "
            "'aldryn doctor'.", fg='red'
        )
        exit(1)

    localdev.create_workspace(obj, slug, path)


@project.group(name='pull')
def project_pull():
    """Pull db or files from Aldryn"""
    pass


@project_pull.command(name='db')
@click.pass_obj
def pull_db(obj):
    localdev.pull_db(obj)


@project_pull.command(name='media')
@click.pass_obj
def pull_media(obj):
    localdev.pull_media(obj)


@project.group(name='push')
def project_push():
    """Push db or media files to Aldryn"""
    pass


@project_push.command(name='db')
@click.pass_obj
def push_db(obj):
    warning = (
        'WARNING',
        '=======',

        '\nYou are about to push your local database to the test server on ',
        'Aldryn. This will replace ALL data on the Aldryn test server with ',
        'the data you are about to push, including (but not limited to):',
        '  - User accounts',
        '  - CMS Pages & Plugins',

        '\nYou will also lose any changes that have been made on the test ',
        'server since you pulled its database to your local environment. ',

        '\nIt is recommended to go the project settings on control.aldryn.com',
        'and take a backup before restoring the database. You can find this ',
        'action in the "Manage Project" section.',

        '\nPlease proceed with caution!'
    )

    click.secho(os.linesep.join(warning), fg='red')
    if not click.confirm('\nAre you sure you want to continue?'):
        return
    localdev.push_db(obj)


@project_push.command(name='media')
@click.pass_obj
def push_media(obj):
    warning = (
        'WARNING',
        '=======',

        '\nYou are about to push your local media files to the test server on ',
        'Aldryn. This will replace ALL existing media files with the ones you ',
        'are about to push.',

        '\nYou will also lose any changes that have been made on the test ',
        'server since you pulled its files to your local environment. ',

        '\nIt is recommended to go the project settings on control.aldryn.com',
        'and take a backup before restoring media files. You can find this ',
        'action in the "Manage Project" section.',

        '\nPlease proceed with caution!'
    )

    click.secho(os.linesep.join(warning), fg='red')
    if not click.confirm('\nAre you sure you want to continue?'):
        return
    localdev.push_media(obj)


@project.command(name='develop')
@click.argument('package')
@click.option(
    '--no-rebuild', is_flag=True, default=False, help='Addon directory'
)
@click.pass_obj
def project_develop(obj, package, no_rebuild):
    """Add a package 'package' to your local project environment"""
    localdev.develop_package(package, no_rebuild)


@cli.group()
@click.option('-p', '--path', default='.', help='Addon directory')
@click.pass_obj
def addon(obj, path):
    """Validate and upload addon packages to Aldryn"""
    pass


@addon.command(name='validate')
@click.pass_context
def addon_validate(ctx):
    """Validate addon configuration"""
    validate_addon(ctx.parent.params['path'])
    click.echo('Addon is valid!')


@addon.command(name='upload')
@click.pass_context
def addon_upload(ctx):
    """Upload addon to Aldryn"""
    ret = upload_addon(ctx.obj, ctx.parent.params['path'])
    click.echo(ret)


@addon.command(name='register')
@click.argument('verbose_name')
@click.argument('package_name')
@click.option('-o', '--organisation', help='Register for an organisation', type=int)
@click.pass_context
def addon_register(ctx, package_name, verbose_name, organisation):
    """Register your addon on Aldryn\n
    - Verbose Name:        Name of the Addon as it appears in the Marketplace.
    - Package Name:        System wide unique Python package name
    """
    ret = ctx.obj.register_addon(package_name, verbose_name, organisation)
    click.echo(ret)


@cli.group()
@click.option('-p', '--path', default='.', help='Boilerplate directory')
@click.pass_obj
def boilerplate(obj, path):
    """Validate and upload boilerplate packages to Aldryn"""
    pass


@boilerplate.command(name='validate')
@click.pass_context
def boilerplate_validate(ctx):
    """Validate boilerplate configuration"""
    validate_boilerplate(ctx.parent.params['path'])
    click.echo('Boilerplate is valid!')


@boilerplate.command(name='upload')
@click.pass_context
def boilerplate_upload(ctx):
    """Upload boilerplate to Aldryn"""
    ret = upload_boilerplate(ctx.obj, ctx.parent.params['path'])
    click.echo(ret)


@cli.command()
@click.option(
    '-s', '--skip-check',  is_flag=True, default=False,
    help="don't check PyPi for newer version",
)
@click.option(
    '-e', '--show-error',  is_flag=True, default=False,
    help="show error if PyPi check fails",
)
def version(skip_check, show_error):
    """Show version info"""
    from . import __version__
    click.echo('package version: {}'.format(__version__))

    # try to get git revision
    script_home = os.path.dirname(__file__)
    git_dir = os.path.join(script_home, '..', '.git')
    if os.path.exists(git_dir):
        revision = subprocess.check_output([
            'git', '--git-dir', git_dir,
            'rev-parse', '--short', 'HEAD'
        ]).strip()
        click.echo('git revision:    {}'.format(revision))

    if not skip_check:
        # check pypi for a newer version
        current_version = StrictVersion(__version__)
        newest_version, exc = get_latest_version_from_pypi()

        if newest_version and newest_version == current_version:
            click.echo('\nYou have the latest version of aldryn-client!')

        elif newest_version and newest_version > current_version:
            click.echo(
                "\nNew version ({newest_version}) available on PyPi. Update "
                "now using 'pip install aldryn-client=={newest_version}'"
                .format(newest_version=newest_version)
            )
        elif newest_version is False:
            if show_error:
                click.secho(
                    'There was an error while trying to check for the latest '
                    'version on pypi.python.org',
                    fg='red'
                )
                if exc:
                    click.echo(exc)


@cli.command()
def doctor():
    """Check if your system meets the requirements
    for Aldryn local development"""
    click.echo('Verifying your system setup')
    exit(0 if check_requirements() else 1)



# -*- coding: utf-8 -*-
import os
import subprocess

import click

from . import utils


def check_requirements(silent=False):
    checks = [
        ('git client', ['git', '--version']),
        ('docker client', ['docker', '--version']),
        ('docker server connection', ['docker', 'ps']),
        ('docker-compose', ['docker-compose', '--version']),
    ]

    errors = []

    for check, cmd in checks:
        error_msg = None
        try:
            utils.check_call(cmd, catch=False, silent=True)
        except OSError as exc:
            if exc.errno == os.errno.ENOENT:
                error_msg = 'executable {} not found'.format(cmd[0])
            else:
                error_msg = (
                    'unknown error while trying to run {}: {}'
                    .format(cmd, exc.message)
                )
        except subprocess.CalledProcessError as exc:
            error_msg = exc.output or str(exc)
        finally:
            if error_msg:
                errors.append((check, error_msg))

            if not silent:
                is_windows = utils.is_windows()
                if error_msg:
                    symbol = ' ERR ' if is_windows else ' ✖  '
                    color = 'red'
                else:
                    symbol = ' OK  ' if is_windows else ' ✓  '
                    color = 'green'
                click.secho(symbol, fg=color, nl=False)
                click.secho('{}'.format(check))

    if errors and not silent:
        click.secho('\nThe following errors happened:', fg='red')
        for error, msg in errors:
            click.secho(' {}:\n > {}'.format(error, msg))

    return not bool(errors)



#!/usr/bin/env python
import sys

from setuptools import setup, find_packages

from aldryn_client import __version__


INSTALL_REQUIRES = [
    'click',
    'requests',
    'tabulate',
    'six',
]

if sys.platform == 'win32':
    INSTALL_REQUIRES += [
        'pyyaml',  # converting docker-compose configs
        'colorama',  # colored output
    ]

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Topic :: Software Development',
]

setup(
    name='aldryn-client',
    version=__version__,
    description='The command-line client for the Aldryn Cloud',
    author='Divio AG',
    author_email='aldryn@divio.ch',
    url='http://www.aldryn.com/',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    license='BSD',
    platforms=['OS Independent'],
    install_requires=INSTALL_REQUIRES,
    entry_points="""
    [console_scripts]
    aldryn = aldryn_client.cli:cli
    """,
)



import os
import tarfile

from six import StringIO

from .. import settings
from ..utils import tar_add_stringio
from ..validators.common import load_config, is_valid_file_name
from ..validators.boilerplate import validate_boilerplate
from .common import add_meta_files


def add_boilerplate_files(path, tar, **complex_extra):
    data_fileobj = StringIO()
    tar_add_stringio(tar, data_fileobj, 'data.yaml')
    for key, value in complex_extra.items():
        dirpath = os.path.join(path, key)
        if os.path.exists(dirpath):
            tar.add(key, filter=value)

    config = load_config(settings.BOILERPLATE_CONFIG_FILENAME, path)
    special_files = [
        f for f in config['protected']
        if not any([f.startswith(key) for key in complex_extra.keys()])
    ]
    for special_file in special_files:
        tar.add(special_file)


def filter_template_files(tarinfo):
    if not tarinfo.isfile():
        return tarinfo
    basename = os.path.basename(tarinfo.name)
    ext = os.path.splitext(basename)[1]
    if ext == '.html':
        return tarinfo
    else:
        return None


def filter_static_files(tarinfo):
    if not tarinfo.isfile():
        return tarinfo
    basename = os.path.basename(tarinfo.name)
    if is_valid_file_name(basename):
        return tarinfo
    else:
        return None


def filter_sass_files(tarinfo):
    basename = os.path.basename(tarinfo.name)
    if tarinfo.isfile():
        if is_valid_file_name(basename):
            return tarinfo
        else:
            return None
    elif basename.startswith('.'):
        return None
    else:
        return tarinfo


def create_boilerplate_archive(path):
    data = StringIO()

    with tarfile.open(mode='w:gz', fileobj=data) as tar:
        add_meta_files(tar, path, settings.BOILERPLATE_CONFIG_FILENAME)
        add_boilerplate_files(
            path, tar,
            templates=filter_template_files,
            static=filter_static_files,
            private=filter_sass_files,
        )

    data.seek(0)
    return data


def upload_boilerplate(client, path=None):
    path = path or '.'
    validate_boilerplate(path)
    archive_obj = create_boilerplate_archive(path)
    return client.upload_boilerplate(archive_obj)
